/*
	Software for ATtiny2313 on
	FMCW Synthesizer 9.2GHz board

	by
	Ruben Undheim
	Master thesis 2012 NTNU
*/
#define F_CPU 1000000UL

#include "ad9858control.h"

uint32_t control_reg;

void delay_ms(uint16_t ms) {
        while ( ms )
        {
                _delay_ms(1);
                ms--;
        }
}
void delay_mini(uint16_t mini) {
        while ( mini )
        {
		asm("nop");
                mini--;
        }
}
void send_byte(uint8_t byte){
	uint8_t active;
	uint8_t i;
	for(i=0;i<8;i++){
		active = (0x01 & (byte >> (7-i)));
		if(active == 1){
			PORTB |= DDS_SDIO;
		}
		else if(active == 0){
			PORTB &= ~DDS_SDIO;
		}
		PORTB |= DDS_SCLK;
		PORTB &= ~DDS_SCLK;
	}
}

uint8_t receive_byte(void){
	uint8_t received = 0x00;
	uint8_t active = 0x00;
	uint8_t i;
	for(i=0;i<8;i++){
		delay_ms(1);
		PORTB |= DDS_SCLK;
		delay_ms(1);
		active = ((PINB & DDS_SDIO) != 0);
		PORTB &= ~DDS_SCLK;
		delay_ms(1);
		received &= (active << (7-i));
	}
	return received;
}
void write_short(uint8_t address, uint16_t data){
	DDRB |= DDS_SDIO;
	address &= ~0x80;
#ifdef ACTIVE
	selectchip(1);	
#endif
	send_byte(address);

	send_byte((uint8_t)((data >> 8) & 0xFF));
	send_byte((uint8_t)((data) & 0xFF));

#ifdef ACTIVE
	selectchip(0);
#endif
}
void write_long(uint8_t address, uint32_t data){
	DDRB |= DDS_SDIO;
	address &= ~0x80;
#ifdef ACTIVE
	selectchip(1);	
#endif
	send_byte(address);

	send_byte((uint8_t)((data >> 24) & 0xFF));
	send_byte((uint8_t)((data >> 16) & 0xFF));
	send_byte((uint8_t)((data >> 8) & 0xFF));
	send_byte((uint8_t)((data) & 0xFF));

#ifdef ACTIVE
	selectchip(0);
#endif
}
uint32_t read_long(uint8_t address){
	uint32_t read = 0x00;
	DDRB |= DDS_SDIO; // write read command
	address |= 0x80;
#ifdef ACTIVE
	selectchip(1);	
#endif

	send_byte(address);

	DDRB &= ~DDS_SDIO;  // start reading
	PORTB &= ~DDS_SDIO;
	read |= (((uint32_t)receive_byte() << 24) & 0xFF000000);	
	read |= (((uint32_t)receive_byte() << 16) & 0x00FF0000); 	
	read |= (((uint32_t)receive_byte() << 8) & 0x0000FF00);	
	read |= ((uint32_t)receive_byte() & 0x000000FF);	

#ifdef ACTIVE
	selectchip(0);
#endif
	return read;
}

void selectchip(uint8_t truefalse){
	if(truefalse == 1)
		PORTD &= ~DDS_CS;
	else if(truefalse == 0)
		PORTD |= DDS_CS;
}

void ioreset(){
	PORTB |= DDS_IORESET;
	delay_ms(1);
	PORTB &= ~DDS_IORESET;
	delay_ms(1);
}


void set_bit(uint8_t bitnr, uint8_t active){
	uint32_t mask = ((uint32_t)0x00000001) << (bitnr);
	if(active == 1)
		set_control((control_reg | mask));	
	else
		set_control((control_reg & ~mask));	
}

void set_control(uint32_t long_word){
	write_long(0x00,long_word);
	control_reg = long_word;
}

void set_default(void){
	set_control(0x00000018);
}

void set_profile(uint8_t profile){
	if(profile == 1){
		PORTD = (PORTD & ~DDS_PS0) & ~DDS_PS1;
	}
	else if(profile == 2){
		PORTD = (PORTD | DDS_PS0) & ~DDS_PS1;
	}
	else if(profile == 3){
		PORTD = (PORTD & ~DDS_PS0) | DDS_PS1;
	}
	else if(profile == 4){
		PORTD = (PORTD | DDS_PS0) | DDS_PS1;
	}
}

void toggle_fud(void){
	PORTD |= DDS_FUD;
	PORTD &= ~DDS_FUD;
}

void init(){
	control_reg = 0x00000018;
	DDRB = 0x0f ; /* Enable output on all the port B pins */
	DDRB &= ~DDS_SDO;
	DDRD = 0xfe ; /* Enable output on all the port B pins */
	PORTB = 0x00 ; /* Set them to 0 */
	PORTD = 0x00;
	PORTD |= DDS_CS;
	delay_ms(100);
	ioreset();
}
