/*
	Software for ATtiny2313 on
	FMCW Synthesizer 9.2GHz board

	by
	Ruben Undheim
	Master thesis 2012 NTNU
*/

#ifndef INC_AD9858_H
#define INC_AD9858_H

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define DDS_PS0	_BV(PD2)
#define DDS_PS1	_BV(PD3)
#define DDS_FUD	_BV(PD4)
#define DDS_RESET	_BV(PD5)
#define DDS_CS	_BV(PD6)  //inverted!!!

#define DDS_SCLK	_BV(PB0)
#define DDS_SDIO	_BV(PB1)
#define DDS_SDO	_BV(PB2)
#define DDS_IORESET	_BV(PB3)

#define CTRL_PWRDOWN	2
#define CTRL_2GHZ	6
#define CTRL_PWRMIX	4
#define CTRL_PWRPD	3
#define CTRL_SWEEP	15
#define CTRL_CLRFREQ	20
#define CTRL_AUTOCLR	23

#define ACTIVE 
#define DELAYBIT 1000
// Clear:
// PORTB &= ~DDS_SCLK;
// Set:
// PORTB |= DDS_SCLK;

// bits transferred at rising SCLK edge
// reading transferred on falling edge of SCLK
// LSB first

void delay_ms(uint16_t ms);
void delay_mini(uint16_t mini);
void send_byte(uint8_t byte);
void write_short(uint8_t address, uint16_t data);
void write_long(uint8_t address, uint32_t data);
uint32_t read_long(uint8_t address);
void ioreset(void);
void init(void);
void selectchip(uint8_t truefalse);
uint8_t receive_byte(void);
void set_control(uint32_t long_word);
void set_default(void);
void set_bit(uint8_t bitnr, uint8_t active);
void toggle_fud(void);
void set_profile(uint8_t profile);

extern uint32_t control_reg;

#endif /* INC_AD9858_H */
